import {
	type Editor,
	type MarkdownView,
	type MarkdownFileInfo,
	Notice,
	TFile,
} from "obsidian";
import { EditorCheckCommand } from "./Command";
import { ReadingProgressModal } from "@ui/modals";
import type BookTrackerPlugin from "@src/main";

export class LogReadingProgressCommand extends EditorCheckCommand {
	constructor(private readonly plugin: BookTrackerPlugin) {
		super("log-reading-progress", "Log Reading Progress");
	}

	private getPageCount(file: TFile): number {
		return this.plugin.getBookMetadata(file)?.pageCount ?? 0;
	}

	protected check(
		_editor: Editor,
		ctx: MarkdownView | MarkdownFileInfo
	): boolean {
		return !(
			ctx.file === null ||
			this.plugin.settings.pageCountProperty === "" ||
			this.getPageCount(ctx.file) <= 0
		);
	}

	protected async run(
		_editor: Editor,
		ctx: MarkdownView | MarkdownFileInfo
	): Promise<void> {
		const file = ctx.file!;
		const fileName = file.basename;
		const pageCount = this.getPageCount(file);
		const pageNumber = await ReadingProgressModal.createAndOpen(
			this.plugin.app,
			pageCount
		);

		if (pageNumber <= 0 || pageNumber > pageCount) {
			new Notice(
				`Invalid page number: ${pageNumber}. It must be between 1 and ${pageCount}.`
			);
			return;
		}

		try {
			await this.plugin.readingLog.createEntry(
				fileName,
				pageNumber,
				pageCount
			);
		} catch (error) {
			new Notice(
				`Failed to log reading progress for ${fileName}: ${error}`
			);
			return;
		}

		new Notice(
			`Logged reading progress for ${fileName}: Page ${pageNumber} of ${pageCount}.`
		);
	}
}
